@@ -1,5 +1,5 @@
 package Test::Mock::Wrapper;
-$Test::Mock::Wrapper::VERSION = '0.06';
+$Test::Mock::Wrapper::VERSION = '0.07';
 use strict;
 use warnings;
 use base qw(Exporter);
@@ -17,7 +17,7 @@ Test::Mock::Wrapper
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -130,9 +130,16 @@ sub new {
 	$controll->{__metaclass} = $metaclass;
 	foreach my $method_name ($metaclass->get_method_list){
 	    push @{ $controll->{__wrapped_symbols} }, {name => $method_name, symbol => $metaclass->find_method_by_name($method_name)};
+	    $controll->{__symbols}{$method_name} = $metaclass->find_method_by_name($method_name)->body;
 	    if ($method_name eq 'new') {
 		my $method = $metaclass->remove_method($method_name);
-		$metaclass->add_method($method_name, sub{ return $controll->getObject;});
+		$metaclass->add_method($method_name, sub{
+		    my $copy = $controll->{__options}{recordType} eq 'copy' ? [@_] : clone(@_);
+		    push @{ $controll->{__calls}{new} }, $copy;
+		    my $obj = bless {}, $object;
+		    push @{ $controll->{__instances} }, $obj;
+		    return $obj;
+		});
 		
 	    }else{
 		my $method = $metaclass->remove_method($method_name);
@@ -147,6 +154,7 @@ sub DESTROY {
     my $controll = shift;
     no strict qw(refs);
     no warnings 'redefine', 'prototype';
+    $controll->resetAll;
     if ($controll->{__metaclass}) {
 	foreach my $sym (@{ $controll->{__wrapped_symbols} }){
 	    if ($sym->{symbol}) {
@@ -169,41 +177,6 @@ sub getObject {
     return $self->{__mocked};
 }
 
-=item $wrapper->resetCalls([$method])
-
-This method clears out the memory of calls that have been made, which is usefull if using the same mock wrapper instance
-multiple tests. When called without arguments, all call history is cleared.  With the optional $method argument, only
-history for that method is called.
-
-=cut
-
-sub resetCalls {
-    my($self, $method) = @_;
-    if (defined($method) && length($method)) {
-	$self->{__calls}{$method} = [];
-    }else{
-	$self->{__calls} = {};
-    }
-    return 1;
-}
-
-=item $wrapper->resetMocks([$method])
-
-This method clears out all previously provided mocked methods. Without arguments, all mocks are cleared. With the optional
-$method argument, only mocks for that method are cleared.
-
-=cut
-
-sub resetMocks {
-    my($self, $method) = @_;
-    if (defined($method) && length($method)) {
-	delete $self->{__mocks}{$method};
-    }else{
-	$self->{__mocks} = {};
-    }
-    return 1;
-}
-
 sub _call {
     my $self = shift;
     my $method = shift;
@@ -234,7 +207,13 @@ sub _call {
     else{
 	# We do not have a default, and our mock type is not stub or mock, try to call underlying object.
 	unshift @_, $self->{__object}; 
-	goto &{ ref($self->{__object}).'::'.$method };
+	if ($self->{__metaclass}) {
+	    # Pacakge is mocked with method wrappers, must call the original symbol metaclass
+	    goto &{ $self->{__symbols}{$method} };
+	}else{
+	    goto &{ ref($self->{__object}).'::'.$method };
+	}
+	
     }
 }
 
@@ -376,8 +355,6 @@ This call returns a Test::Mock::Wrapper::Verify object, which can be used to exa
 specified method thus far.  These objects are intended to be used to simplify testing, and methods called on the it
 are I<chainable> to lend to more readable tests.
 
-=back
-
 =cut
 
 sub verify {
@@ -386,8 +363,64 @@ sub verify {
 }
 
 
+=item $wrapper->resetCalls([$method])
+
+This method clears out the memory of calls that have been made, which is usefull if using the same mock wrapper instance
+multiple tests. When called without arguments, all call history is cleared.  With the optional $method argument, only
+history for that method is called.
+
+=cut
+
+sub resetCalls {
+    my($self, $method) = @_;
+    if (defined($method) && length($method)) {
+	$self->{__calls}{$method} = [];
+    }else{
+	$self->{__calls} = {};
+    }
+    return 1;
+}
+
+=item $wrapper->resetMocks([$method])
+
+This method clears out all previously provided mocked methods. Without arguments, all mocks are cleared. With the optional
+$method argument, only mocks for that method are cleared.
+
+=cut
+
+sub resetMocks {
+    my($self, $method) = @_;
+    if (defined($method) && length($method)) {
+	delete $self->{__mocks}{$method};
+    }else{
+	$self->{__mocks} = {};
+    }
+    return 1;
+}
+
+=item $wrapper->resetAll
+
+This method clears out both mocks and calls.  Will also rebless any mocked instances created from a mocked package
+(Prevents intermitent failures during global destruction).
+
+=back
+
+=cut
+
+sub resetAll {
+    my $self = shift;
+    if ($self->{__metaclass}) {
+        foreach my $inst (@{ $self->{__instances} }){
+            bless $inst, 'Test::Mock::Wrapped' if($inst);
+        }
+    }
+    $self->{__instances} = [];
+    $self->{__calls}     = {};
+    $self->{__mocks}     = {};
+}
+
 package Test::Mock::Wrapped;
-$Test::Mock::Wrapped::VERSION = '0.06';
+$Test::Mock::Wrapped::VERSION = '0.07';
 use Carp;
 use Scalar::Util qw(weaken isweak);
 use vars qw(@ISA);
